#include "map.h"

#define PSF .8
#define VMARGIN 5 /* margin at top and bottom of legend graphic */
#define HMARGIN 5 /* margin at left and right of legend graphic */

/*
** Creates a GD image of a legend for a specific map. msDrawLegend()
** respects the current scale, and classes without a name are not
** added to the legend.
*/

gdImagePtr msDrawLegend(mapObj *map)
{
  shapeObj p;
  gdImagePtr img; /* image data structure */
  int i,j; /* loop counters */
  pointObj pnt;
  int size_x, size_y;
  double scaleFactor;
  layerObj *lp;  
  int maxwidth=0, maxheight=0, width, height, n=0;
  int *heights;

  map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
  map->scale = msCalculateScale(map->extent, map->units, map->width, map->height);

  /* Initialize the polygon/polyline */
  p.line = (lineObj *)malloc(sizeof(lineObj));
  p.numlines = 1;
  p.line[0].point = (pointObj *)malloc(sizeof(pointObj)*4);
  p.line[0].numpoints = 4;

  /*
  ** allocate heights array
  */
  for(i=0; i<map->numlayers; i++) {
    if((map->layers[i].status == MS_OFF) || (map->layers[i].status == MS_QUERY)) /* skip it */
      continue;
    
    if(map->scale > 0) { /* user or application set a scale */ 
      if((map->layers[i].maxscale > 0) && (map->scale > map->layers[i].maxscale))
	  continue;
      if((map->layers[i].minscale > 0) && (map->scale <= map->layers[i].minscale))
	  continue;
    }

    for(j=0;j<map->layers[i].numclasses;j++) {
      if(map->layers[i].class[j].name == NULL)
	continue; /* skip it */
      n++;
    }
  }

  if((heights = (int *)malloc(sizeof(int)*n)) == NULL) {
    msSetError(MS_MEMERR, "Error allocating heights array.", "msDrawLegend()");
    return(NULL);
  }

  /*
  ** Calculate the optimal image size for the legend
  */
  n=0;
  for(i=0; i<map->numlayers; i++) { /* Need to find the longest legend label string */
    if((map->layers[i].status == MS_OFF) || (map->layers[i].status == MS_QUERY)) /* skip it */
      continue;
    
    if(map->scale > 0) { /* user or application set a scale */ 
      if((map->layers[i].maxscale > 0) && (map->scale > map->layers[i].maxscale))
	  continue;
      if((map->layers[i].minscale > 0) && (map->scale <= map->layers[i].minscale))
	  continue;
    }

    for(j=0;j<map->layers[i].numclasses;j++) {
      if(map->layers[i].class[j].name == NULL)
	continue; /* skip it */
      if(msGetLabelSize(map->layers[i].class[j].name, &map->legend.label, &width, &height, &(map->fontset)) != 0)
	return(NULL); /* something bad happened */
      maxheight = MS_MAX(maxheight, height);
      maxwidth = MS_MAX(maxwidth, width);
      heights[n] = height;
      n++;
    }
  }
  
  size_x = (2*HMARGIN)+(maxwidth)+(map->legend.keyspacingx)+(map->legend.keysizex);
  size_y = (2*VMARGIN) + ((n-1)*map->legend.keyspacingy);
  for(i=0; i<n; i++) {
    heights[i] = MS_MAX(heights[i], maxheight);
    size_y += MS_MAX(heights[i], map->legend.keysizey);
  }

  /*
  ** Initialize the legend image
  */
  if((img = gdImageCreate(size_x, size_y)) == NULL) {
    msSetError(MS_GDERR, "Unable to initialize image.", "msDrawLegend()");
    free(heights);
    return(NULL);
  }

  /*
  ** Load colormap for the image
  */
  if(msLoadPalette(img, &map->palette, map->legend.imagecolor) == -1)
     return(NULL);

  pnt.y = VMARGIN;
    
  for(i=0; i<map->numlayers; i++) { /* For each layer */

    lp = &(map->layers[i]); /* assign for brevity */

    if((lp->numclasses == 0) || (lp->status == MS_OFF) || (lp->status == MS_QUERY))
      continue; /* skip this layer */

    if(map->scale > 0) { /* user or application set a scale */ 
      if((lp->maxscale > 0) && (map->scale > lp->maxscale))
	  continue;
      if((lp->minscale > 0) && (map->scale <= lp->minscale))
	  continue;
    }

    if((map->scale != -1) && (lp->symbolscale != -1)) /* some legend chips can contain scaled symbols */
       scaleFactor = lp->symbolscale/map->scale;
    else
       scaleFactor = 1.0;

    for(j=0; j<lp->numclasses; j++) { /* always at least 1 class */

      if(lp->class[j].name == NULL)
	continue; /* skip it */
      
      pnt.x = HMARGIN + map->legend.keysizex + map->legend.keyspacingx;

      lp->class[i].size = lp->class[i].size*scaleFactor; /* change size accordingly */

      /* 
      ** now draw the appropriate color/symbol/size combination 
      */      
      switch(lp->type) {
        case MS_POINT:            
          p.line[0].point[0].x = MS_NINT(HMARGIN + (map->legend.keysizex/2.0)) - 1;
          p.line[0].point[0].y = MS_NINT(pnt.y + (map->legend.keysizey/2.0)) - 1;
	  p.line[0].numpoints = 1;
	  msDrawMarkerSymbol(&(map->markerset), img, &(p.line[0].point[0]), &(lp->class[j]));
          break;
        case MS_LINE:
	case MS_POLYLINE:
          p.line[0].point[0].x = HMARGIN;
          p.line[0].point[0].y = pnt.y + map->legend.keysizey - 1;
	  p.line[0].point[1].x = HMARGIN + MS_NINT(map->legend.keysizex/3.0) - 1;
          p.line[0].point[1].y = pnt.y;
          p.line[0].point[2].x = HMARGIN + MS_NINT(2*map->legend.keysizex/3.0) - 1;
          p.line[0].point[2].y = pnt.y + map->legend.keysizey - 1;
	  p.line[0].point[3].x = HMARGIN + map->legend.keysizex - 1;
          p.line[0].point[3].y = pnt.y;
	  p.line[0].numpoints = 4;
          msDrawLineSymbol(&(map->lineset), img, &p, &(lp->class[j]));       
	  break;
        case MS_RASTER:
        case MS_POLYGON:
	  p.line[0].point[0].x = HMARGIN;
          p.line[0].point[0].y = pnt.y;
	  p.line[0].point[1].x = HMARGIN + map->legend.keysizex - 1;
          p.line[0].point[1].y = pnt.y;
          p.line[0].point[2].x = HMARGIN + map->legend.keysizex - 1;
          p.line[0].point[2].y = pnt.y + map->legend.keysizey - 1;
	  p.line[0].point[3].x = HMARGIN;
          p.line[0].point[3].y = pnt.y + map->legend.keysizey - 1;
	  p.line[0].numpoints = 4;
	  msDrawShadeSymbol(&(map->shadeset), img, &p, &(lp->class[j]));
          break;
        case MS_ANNOTATION:        
          break;
     } /* end symbol drawing */

      /* Draw the outline if a color is specified */
      if(map->legend.outlinecolor > 0) { /* 0 is background, so who cares about drawing it. */
	p.line[0].point[0].x = HMARGIN;
        p.line[0].point[0].y = pnt.y;
	p.line[0].point[1].x = HMARGIN + map->legend.keysizex - 1;
        p.line[0].point[1].y = pnt.y;
        p.line[0].point[2].x = HMARGIN + map->legend.keysizex - 1;
        p.line[0].point[2].y = pnt.y + map->legend.keysizey - 1;
        p.line[0].point[3].x = HMARGIN;
        p.line[0].point[3].y = pnt.y + map->legend.keysizey - 1;
	msImagePolygon(img, &p, map->legend.outlinecolor);
      }

      pnt.y += MS_MAX(map->legend.keysizey, maxheight);
      msDrawLabel(img, map, pnt, lp->class[j].name, &(map->legend.label));

      pnt.y += map->legend.keyspacingy; /* bump y for next label */
	
    } /* next label */
  } /* next layer please */

  free(heights);
  free(p.line[0].point);
  free(p.line);

  return(img);  /* return image */
}
