#include "map.h"

extern int bBigEndian;

/*
** Utility functions from shapelib.
*/
static void SwapWord( int length, void * wordP )
{
  int i;
  uchar	temp;
  
  for( i=0; i < length/2; i++ )
    {
      temp = ((uchar *) wordP)[i];
      ((uchar *)wordP)[i] = ((uchar *) wordP)[length-i-1];
      ((uchar *) wordP)[length-i-1] = temp;
    }
}

static void * SfRealloc( void * pMem, int nNewSize )     
{
  if( pMem == NULL )
    return( (void *) malloc(nNewSize) );
  else
    return( (void *) realloc(pMem,nNewSize) );
}

static void writeBounds( uchar * pabyRec, shapeObj *shape, int nVCount )
{
    double	dXMin, dXMax, dYMin, dYMax;
    int		i, j;

    if( nVCount == 0 ) {
      dXMin = dYMin = dXMax = dYMax = 0.0;
    } else {
      dXMin = dXMax = shape->line[0].point[0].x;
      dYMin = dYMax = shape->line[0].point[0].y;
      
      for( i=0; i<shape->numlines; i++ ) {
	for( j=0; j<shape->line[i].numpoints; j++ ) {
	  dXMin = MS_MIN(dXMin, shape->line[i].point[j].x);
	  dXMax = MS_MAX(dXMax, shape->line[i].point[j].x);
	  dYMin = MS_MIN(dYMin, shape->line[i].point[j].y);
	  dYMax = MS_MAX(dYMax, shape->line[i].point[j].y);
	}
      }
    }

    if( bBigEndian ) { 
      SwapWord( 8, &dXMin );
      SwapWord( 8, &dYMin );
      SwapWord( 8, &dXMax );
      SwapWord( 8, &dYMax );
    }

    ByteCopy( &dXMin, pabyRec +  0, 8 );
    ByteCopy( &dYMin, pabyRec +  8, 8 );
    ByteCopy( &dXMax, pabyRec + 16, 8 );
    ByteCopy( &dYMax, pabyRec + 24, 8 );
}

int msWriteShape(SHPHandle psSHP, shapeObj *shape )
{
    int	       	nRecordOffset, i, j, k, nRecordSize=0;
    uchar	*pabyRec;
    shp_int32	i32, nPoints, nParts;

    psSHP->bUpdated = TRUE;

    /* -------------------------------------------------------------------- */
    /*      Add the new entity to the in memory index.                      */
    /* -------------------------------------------------------------------- */
    psSHP->nRecords++;
    if( psSHP->nRecords > psSHP->nMaxRecords )
    {
	psSHP->nMaxRecords = psSHP->nMaxRecords * 1.3 + 100;

	psSHP->panRecOffset = (int *) 
	  SfRealloc(psSHP->panRecOffset,sizeof(int) * psSHP->nMaxRecords );
	psSHP->panRecSize = (int *) 
	  SfRealloc(psSHP->panRecSize,sizeof(int) * psSHP->nMaxRecords );
    }

    /* -------------------------------------------------------------------- */
    /*      Compute a few things.                                           */
    /* -------------------------------------------------------------------- */
    nPoints = 0;
    for(i=0; i<shape->numlines; i++)
      nPoints += shape->line[i].numpoints;
    
    nParts = shape->numlines;

    /* -------------------------------------------------------------------- */
    /*      Initialize record.                                              */
    /* -------------------------------------------------------------------- */
    psSHP->panRecOffset[psSHP->nRecords-1] = nRecordOffset = psSHP->nFileSize;
    
    pabyRec = (uchar *) malloc(nPoints * 2 * sizeof(double) + nParts * 4 + 128);

    
    /* -------------------------------------------------------------------- */
    /*  Write vertices for a Polygon or Arc.				    */
    /* -------------------------------------------------------------------- */
    if( psSHP->nShapeType == SHPT_POLYGON || psSHP->nShapeType == SHPT_ARC ) {
      shp_int32 t_nParts, t_nPoints, partSize;
      
      t_nParts = nParts;
      t_nPoints = nPoints;

      writeBounds( pabyRec + 12, shape, t_nPoints );
      
      if( bBigEndian ) { 
	SwapWord( 4, &nPoints );
	SwapWord( 4, &nParts );
      }
      
      ByteCopy( &nPoints, pabyRec + 40 + 8, 4 );
      ByteCopy( &nParts, pabyRec + 36 + 8, 4 );
      
      for( i = 0; i < t_nParts; i++ ) {

	if( i == t_nParts-1)
	  partSize = t_nPoints - shape->line[i].numpoints;
	else
	  partSize = shape->line[i+1].numpoints - shape->line[i].numpoints;

	ByteCopy( &partSize, pabyRec + 44 + 8 + 4*i, 4 );
	if( bBigEndian ) SwapWord( 4, pabyRec + 44 + 8 + 4*i);
      }
      
      k = 0; // overall point counter
      for( i = 0; i < shape->numlines; i++ ) {
	for( j = 0; j < shape->line[i].numpoints; j++ ) {
	  ByteCopy( &(shape->line[i].point[j].x), pabyRec + 44 + 4*t_nParts + 8 + k * 16, 8 );
	  ByteCopy( &(shape->line[i].point[j].y), pabyRec + 44 + 4*t_nParts + 8 + k * 16 + 8, 8 );
	  
	  if( bBigEndian ) {
	    SwapWord( 8, pabyRec + 44+4*t_nParts+8+k*16 );
	    SwapWord( 8, pabyRec + 44+4*t_nParts+8+k*16+8 );
	  }
	  
	  k++;
	}
      }
      
      nRecordSize = 44 + 4*t_nParts + 16 * t_nPoints;
    }

    /* -------------------------------------------------------------------- */
    /*  Write vertices for a MultiPoint.				    */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_MULTIPOINT )
    {
      shp_int32 t_nPoints;
      
      t_nPoints = nPoints;
      
      writeBounds( pabyRec + 12, shape, nPoints );
      
      if( bBigEndian ) SwapWord( 4, &nPoints );
      ByteCopy( &nPoints, pabyRec + 44, 4 );
      
      for( i = 0; i < shape->line[0].numpoints; i++ ) {
	ByteCopy( &(shape->line[0].point[i].x), pabyRec + 48 + i*16, 8 );
	ByteCopy( &(shape->line[0].point[i].y), pabyRec + 48 + i*16 + 8, 8 );
	
	if( bBigEndian ) { 
	  SwapWord( 8, pabyRec + 48 + i*16 );
	  SwapWord( 8, pabyRec + 48 + i*16 + 8 );
	}
      }
      
      nRecordSize = 40 + 16 * t_nPoints;
    }

    /* -------------------------------------------------------------------- */
    /*      Write vertices for a point.                                     */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_POINT )
    {
	ByteCopy( &(shape->line[0].point[0].x), pabyRec + 12, 8 );
	ByteCopy( &(shape->line[0].point[0].y), pabyRec + 20, 8 );

	if( bBigEndian ) {
	  SwapWord( 8, pabyRec + 12 );
	  SwapWord( 8, pabyRec + 20 );
	}

	nRecordSize = 20;
    }

    /* -------------------------------------------------------------------- */
    /*      Set the shape type, record number, and record size.             */
    /* -------------------------------------------------------------------- */
    i32 = psSHP->nRecords-1+1;					/* record # */
    if( !bBigEndian ) SwapWord( 4, &i32 );
    ByteCopy( &i32, pabyRec, 4 );

    i32 = nRecordSize/2;				/* record size */
    if( !bBigEndian ) SwapWord( 4, &i32 );
    ByteCopy( &i32, pabyRec + 4, 4 );

    i32 = psSHP->nShapeType;				/* shape type */
    if( bBigEndian ) SwapWord( 4, &i32 );
    ByteCopy( &i32, pabyRec + 8, 4 );

    /* -------------------------------------------------------------------- */
    /*      Write out record.                                               */
    /* -------------------------------------------------------------------- */
    fseek( psSHP->fpSHP, nRecordOffset, 0 );
    fwrite( pabyRec, nRecordSize+8, 1, psSHP->fpSHP );
    free( pabyRec );

    psSHP->panRecSize[psSHP->nRecords-1] = nRecordSize;
    psSHP->nFileSize += nRecordSize + 8;

    /* -------------------------------------------------------------------- */
    /*	Expand file wide bounds based on this shape.			    */
    /* -------------------------------------------------------------------- */
    if( psSHP->nRecords == 1 ) {
	psSHP->adBoundsMin[0] = psSHP->adBoundsMax[0] = shape->line[0].point[0].x;
	psSHP->adBoundsMin[1] = psSHP->adBoundsMax[1] = shape->line[0].point[0].y;
    }

    for( i=0; i<shape->numlines; i++ ) {
      for( j=0; j<shape->line[i].numpoints; j++ ) {
	psSHP->adBoundsMin[0] = MS_MIN(psSHP->adBoundsMin[0], shape->line[i].point[j].x);
	psSHP->adBoundsMin[1] = MS_MIN(psSHP->adBoundsMin[1], shape->line[i].point[j].x);
	psSHP->adBoundsMax[0] = MS_MAX(psSHP->adBoundsMax[0], shape->line[i].point[j].y);
	psSHP->adBoundsMax[1] = MS_MAX(psSHP->adBoundsMax[1], shape->line[i].point[j].y);
      }
    }

    return( psSHP->nRecords - 1 );
}

#ifdef USE_PROJ

/*
** msReadShapeProj() - Reads the vertices for one shape from a shape file. Swiped
** from the shapelib code and adapted for mapserver feature abstractions. Only
** supports polygon and line features. Projection is done as points are moved into
** the polygon/polyline stucture.
*/
void msReadShapeProj( SHPHandle psSHP, int hEntity, shapeObj *shape, projectionObj *in, projectionObj *out )
{
    int	       		j, k;
    static uchar	*pabyRec = NULL;   
    static int		nPartMax = 0, *panParts = NULL;
    static int		nBufSize = 0;

    /* -------------------------------------------------------------------- */
    /*      Validate the record/entity number.                              */
    /* -------------------------------------------------------------------- */
    if( hEntity < 0 || hEntity >= psSHP->nRecords ) {
      shape->line = NULL;
      shape->numlines = 0;
      return;
    }

    /* -------------------------------------------------------------------- */
    /*      Ensure our record buffer is large enough.                       */
    /* -------------------------------------------------------------------- */
    if( psSHP->panRecSize[hEntity]+8 > nBufSize )
    {
	nBufSize = psSHP->panRecSize[hEntity]+8;
	pabyRec = (uchar *) SfRealloc(pabyRec,nBufSize);
    }    

    /* -------------------------------------------------------------------- */
    /*      Read the record.                                                */
    /* -------------------------------------------------------------------- */
    fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity], 0 );
    fread( pabyRec, psSHP->panRecSize[hEntity]+8, 1, psSHP->fpSHP );
	                        
    /* -------------------------------------------------------------------- */
    /*  Extract vertices for a Polygon or Arc.				*/
    /* -------------------------------------------------------------------- */
    if( psSHP->nShapeType == SHPT_POLYGON || psSHP->nShapeType == SHPT_ARC )
    {
      shp_int32		nPoints, nParts;
      int    		i;
      
      memcpy( &nPoints, pabyRec + 40 + 8, 4 );
      memcpy( &nParts, pabyRec + 36 + 8, 4 );
      
      if( bBigEndian ) {
	SwapWord( 4, &nPoints );
        SwapWord( 4, &nParts );
      }

      /* -------------------------------------------------------------------- */
      /*      Copy out the part array from the record.                        */
      /* -------------------------------------------------------------------- */
      if( nPartMax < nParts )
      {
	nPartMax = nParts;
	panParts = (int *) SfRealloc(panParts, nPartMax * sizeof(int) );
      }
      
      memcpy( panParts, pabyRec + 44 + 8, 4 * nParts );
      for( i = 0; i < nParts; i++ )
      {
	if( bBigEndian ) SwapWord( 4, panParts+i );
      }
      
      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj)*nParts)) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }

      shape->numlines = nParts;
      
      k = 0; /* overall point counter */

      if((in->numargs > 0) && (out->numargs > 0)) { /* projection needed */
	for( i = 0; i < nParts; i++) {
	  if( i == nParts-1)
	    shape->line[i].numpoints = nPoints - panParts[i];
	  else
	    shape->line[i].numpoints = panParts[i+1] - panParts[i];
	  
	  if( (shape->line[i].point = (pointObj *)malloc(sizeof(pointObj)*shape->line[i].numpoints)) == NULL ) {
	    shape->line = NULL;
	    shape->numlines = 0;
	    return;
	  }
	  
	  for( j = 0; j < shape->line[i].numpoints; j++ ) {
	    memcpy(&(shape->line[i].point[j].x), pabyRec + 44 + 4*nParts + 8 + k * 16, 8 );
	    memcpy(&(shape->line[i].point[j].y), pabyRec + 44 + 4*nParts + 8 + k * 16 + 8, 8 );
	    
	    if( bBigEndian ) {
	      SwapWord( 8, &(shape->line[i].point[j].x) );
	      SwapWord( 8, &(shape->line[i].point[j].y) );
	    }

	    msProjectPoint(in->proj, out->proj, &(shape->line[i].point[j]));

	    k++;
	  }
	}
      } else {
	for( i = 0; i < nParts; i++) { 	  
	  if( i == nParts-1)
	    shape->line[i].numpoints = nPoints - panParts[i];
	  else
	    shape->line[i].numpoints = panParts[i+1] - panParts[i];
	  
	  if( (shape->line[i].point = (pointObj *)malloc(sizeof(pointObj)*shape->line[i].numpoints)) == NULL ) {
	    shape->line = NULL;
	    shape->numlines = 0;
	    return;
	  }
	  
	  for( j = 0; j < shape->line[i].numpoints; j++ ) {
	    memcpy(&(shape->line[i].point[j].x), pabyRec + 44 + 4*nParts + 8 + k * 16, 8 );
	    memcpy(&(shape->line[i].point[j].y), pabyRec + 44 + 4*nParts + 8 + k * 16 + 8, 8 );
	    
	    if( bBigEndian ) {
	      SwapWord( 8, &(shape->line[i].point[j].x) );
	      SwapWord( 8, &(shape->line[i].point[j].y) );
	    }
	   
	    k++;
	  }
	}
      }
    } 

    /* -------------------------------------------------------------------- */
    /*  Extract a MultiPoint.   			                    */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_MULTIPOINT)
    {
      shp_int32		nPoints;
      int    		i;
    
      memcpy( &nPoints, pabyRec + 44, 4 );
      if( bBigEndian ) SwapWord( 4, &nPoints );
    
      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj))) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }

      shape->numlines = 1;
      shape->line[0].numpoints = nPoints;
      shape->line[0].point = (pointObj *) malloc( nPoints * sizeof(pointObj) );
      
      if((in->numargs > 0) && (out->numargs > 0)) {
	for( i = 0; i < nPoints; i++ ) {
	  memcpy(&(shape->line[0].point[i].x), pabyRec + 48 + 16 * i, 8 );
	  memcpy(&(shape->line[0].point[i].y), pabyRec + 48 + 16 * i + 8, 8 );
	  
	  if( bBigEndian ) {
	    SwapWord( 8, &(shape->line[0].point[i].x) );
	    SwapWord( 8, &(shape->line[0].point[i].y) );
	  }
	  
	  msProjectPoint(in->proj, out->proj, &shape->line[0].point[i]);
	}
      } else {
	for( i = 0; i < nPoints; i++ ) {
	  memcpy(&(shape->line[0].point[i].x), pabyRec + 48 + 16 * i, 8 );
	  memcpy(&(shape->line[0].point[i].y), pabyRec + 48 + 16 * i + 8, 8 );
	  
	  if( bBigEndian ) {
	    SwapWord( 8, &(shape->line[0].point[i].x) );
	    SwapWord( 8, &(shape->line[0].point[i].y) );
	  }
	}
      }
    }

    /* -------------------------------------------------------------------- */
    /*  Extract a Point.   			                    */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_POINT )
    {    

      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj))) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }
      
      shape->numlines = 1;
      shape->line[0].numpoints = 1;
      shape->line[0].point = (pointObj *) malloc(sizeof(pointObj));
      
      memcpy( &(shape->line[0].point[0].x), pabyRec + 12, 8 );
      memcpy( &(shape->line[0].point[0].y), pabyRec + 20, 8 );
      
      if( bBigEndian ) {
	SwapWord( 8, &(shape->line[0].point[0].x));
	SwapWord( 8, &(shape->line[0].point[0].y));
      }
      
      if((in->numargs > 0) && (out->numargs > 0))
	msProjectPoint(in->proj, out->proj, &shape->line[0].point[0]);

    } else {
      shape->line = NULL;
      shape->numlines = 0;
    }    

    return;
}

void msReadBoundsProj( SHPHandle psSHP, int hEntity, rectObj *padBounds, projectionObj *in, projectionObj *out)
{
   /* -------------------------------------------------------------------- */
   /*      Validate the record/entity number.                              */
   /* -------------------------------------------------------------------- */
    if( hEntity < -1 || hEntity >= psSHP->nRecords )
    {
	padBounds->minx = padBounds->miny = padBounds->maxx = padBounds->maxy = 0.0;
        return;
    }

    /* -------------------------------------------------------------------- */
    /*	If the entity is -1 we fetch the bounds for the whole file.	    */
    /* -------------------------------------------------------------------- */
    if( hEntity == -1 )
    {
	padBounds->minx = psSHP->adBoundsMin[0];
	padBounds->miny = psSHP->adBoundsMin[1];
	padBounds->maxx = psSHP->adBoundsMax[0];
	padBounds->maxy = psSHP->adBoundsMax[1];
    }

    /* -------------------------------------------------------------------- */
    /*      Extract bounds for any record but a point record.               */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType != SHPT_POINT )
    {
	fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity]+12, 0 );
	fread( padBounds, sizeof(double)*4, 1, psSHP->fpSHP );

	if( bBigEndian )
	{
	    SwapWord( 8, &(padBounds->minx) );
	    SwapWord( 8, &(padBounds->miny) );
	    SwapWord( 8, &(padBounds->maxx) );
	    SwapWord( 8, &(padBounds->maxy) );
	}
    }

    /* -------------------------------------------------------------------- */
    /*      For points we fetch the point, and duplicate it as the          */
    /*      minimum and maximum bound.                                      */
    /* -------------------------------------------------------------------- */
    else
    {
	fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity]+12, 0 );
	fread( padBounds, sizeof(double)*2, 1, psSHP->fpSHP );

	if( bBigEndian ) {
	    SwapWord( 8, &(padBounds->minx) );
	    SwapWord( 8, &(padBounds->miny) );
	}

	padBounds->maxx = padBounds->minx;
	padBounds->maxy = padBounds->miny;
    }
    
    if((in->numargs > 0) && (out->numargs > 0)) /* project the bounding box */
      msProjectRect(in->proj, out->proj, padBounds);
}

#endif

/*
** msReadShape() - Reads the vertices for one shape from a shape file. Swiped
** from the shapelib code and adapted for mapserver feature primitives.
*/
void msReadShape( SHPHandle psSHP, int hEntity, shapeObj *shape )
{
    int	       		j, k;
    static uchar	*pabyRec = NULL;   
    static int		nPartMax = 0, *panParts = NULL;
    static int		nBufSize = 0;

    /* -------------------------------------------------------------------- */
    /*      Validate the record/entity number.                              */
    /* -------------------------------------------------------------------- */
    if( hEntity < 0 || hEntity >= psSHP->nRecords ) {
      shape->line = NULL;
      shape->numlines = 0;
      return;
    }

    /* -------------------------------------------------------------------- */
    /*      Ensure our record buffer is large enough.                       */
    /* -------------------------------------------------------------------- */
    if( psSHP->panRecSize[hEntity]+8 > nBufSize )
    {
	nBufSize = psSHP->panRecSize[hEntity]+8;
	pabyRec = (uchar *) SfRealloc(pabyRec,nBufSize);
    }    

    /* -------------------------------------------------------------------- */
    /*      Read the record.                                                */
    /* -------------------------------------------------------------------- */
    fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity], 0 );
    fread( pabyRec, psSHP->panRecSize[hEntity]+8, 1, psSHP->fpSHP );
	                        
    /* -------------------------------------------------------------------- */
    /*  Extract vertices for a Polygon or Arc.				*/
    /* -------------------------------------------------------------------- */
    if( psSHP->nShapeType == SHPT_POLYGON || psSHP->nShapeType == SHPT_ARC )
    {
      shp_int32		nPoints, nParts;
      int    		i;
      
      memcpy( &nPoints, pabyRec + 40 + 8, 4 );
      memcpy( &nParts, pabyRec + 36 + 8, 4 );
      
      if( bBigEndian ) {
	SwapWord( 4, &nPoints );
        SwapWord( 4, &nParts );
      }

      /* -------------------------------------------------------------------- */
      /*      Copy out the part array from the record.                        */
      /* -------------------------------------------------------------------- */
      if( nPartMax < nParts )
      {
	nPartMax = nParts;
	panParts = (int *) SfRealloc(panParts, nPartMax * sizeof(int) );
      }
      
      memcpy( panParts, pabyRec + 44 + 8, 4 * nParts );
      for( i = 0; i < nParts; i++ )
	if( bBigEndian ) SwapWord( 4, panParts+i );
      
      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj)*nParts)) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }

      shape->numlines = nParts;
      
      k = 0; /* overall point counter */
      for( i = 0; i < nParts; i++) 
      { 	  
	if( i == nParts-1)
	  shape->line[i].numpoints = nPoints - panParts[i];
	else
	  shape->line[i].numpoints = panParts[i+1] - panParts[i];
	
	if( (shape->line[i].point = (pointObj *)malloc(sizeof(pointObj)*shape->line[i].numpoints)) == NULL ) {
	  shape->line = NULL;
	  shape->numlines = 0;
	  return;
	}

	for( j = 0; j < shape->line[i].numpoints; j++ )
	{
	  memcpy(&(shape->line[i].point[j].x), pabyRec + 44 + 4*nParts + 8 + k * 16, 8 );
	  memcpy(&(shape->line[i].point[j].y), pabyRec + 44 + 4*nParts + 8 + k * 16 + 8, 8 );
	  
	  if( bBigEndian ) {
	    SwapWord( 8, &(shape->line[i].point[j].x) );
	    SwapWord( 8, &(shape->line[i].point[j].y) );
	  }

	  k++;
	}
      }
    } 

    /* -------------------------------------------------------------------- */
    /*  Extract a MultiPoint.   			                    */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_MULTIPOINT)
    {
      shp_int32		nPoints;
      int    		i;
    
      memcpy( &nPoints, pabyRec + 44, 4 );
      if( bBigEndian ) SwapWord( 4, &nPoints );
    
      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj))) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }

      shape->numlines = 1;
      shape->line[0].numpoints = nPoints;
      shape->line[0].point = (pointObj *) malloc( nPoints * sizeof(pointObj) );
      
      for( i = 0; i < nPoints; i++ ) {
	memcpy(&(shape->line[0].point[i].x), pabyRec + 48 + 16 * i, 8 );
	memcpy(&(shape->line[0].point[i].y), pabyRec + 48 + 16 * i + 8, 8 );
	
	if( bBigEndian ) {
	  SwapWord( 8, &(shape->line[0].point[i].x) );
	  SwapWord( 8, &(shape->line[0].point[i].y) );
	}
      }
    }

    /* -------------------------------------------------------------------- */
    /*  Extract a Point.   			                    */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType == SHPT_POINT )
    {    

      /* -------------------------------------------------------------------- */
      /*      Fill the shape structure.                                       */
      /* -------------------------------------------------------------------- */
      if( (shape->line = (lineObj *)malloc(sizeof(lineObj))) == NULL ) {
	shape->line = NULL;
	shape->numlines = 0;
	return;
      }

      shape->numlines = 1;
      shape->line[0].numpoints = 1;
      shape->line[0].point = (pointObj *) malloc(sizeof(pointObj));
      
      memcpy( &(shape->line[0].point[0].x), pabyRec + 12, 8 );
      memcpy( &(shape->line[0].point[0].y), pabyRec + 20, 8 );
      
      if( bBigEndian ) {
	SwapWord( 8, &(shape->line[0].point[0].x));
	SwapWord( 8, &(shape->line[0].point[0].y));
      }

    } else {
      shape->line = NULL;
      shape->numlines = 0;
    }    

    return;
}

void msReadBounds( SHPHandle psSHP, int hEntity, rectObj *padBounds)
{
   /* -------------------------------------------------------------------- */
   /*      Validate the record/entity number.                              */
   /* -------------------------------------------------------------------- */
    if( hEntity < -1 || hEntity >= psSHP->nRecords )
    {
	padBounds->minx = padBounds->miny = padBounds->maxx = padBounds->maxy = 0.0;
        return;
    }

    /* -------------------------------------------------------------------- */
    /*	If the entity is -1 we fetch the bounds for the whole file.	    */
    /* -------------------------------------------------------------------- */
    if( hEntity == -1 )
    {
	padBounds->minx = psSHP->adBoundsMin[0];
	padBounds->miny = psSHP->adBoundsMin[1];
	padBounds->maxx = psSHP->adBoundsMax[0];
	padBounds->maxy = psSHP->adBoundsMax[1];
    }

    /* -------------------------------------------------------------------- */
    /*      Extract bounds for any record but a point record.               */
    /* -------------------------------------------------------------------- */
    else if( psSHP->nShapeType != SHPT_POINT )
    {
	fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity]+12, 0 );
	fread( padBounds, sizeof(double)*4, 1, psSHP->fpSHP );

	if( bBigEndian )
	{
	    SwapWord( 8, &(padBounds->minx) );
	    SwapWord( 8, &(padBounds->miny) );
	    SwapWord( 8, &(padBounds->maxx) );
	    SwapWord( 8, &(padBounds->maxy) );
	}
    }

    /* -------------------------------------------------------------------- */
    /*      For points we fetch the point, and duplicate it as the          */
    /*      minimum and maximum bound.                                      */
    /* -------------------------------------------------------------------- */
    else
    {
	fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity]+12, 0 );
	fread( padBounds, sizeof(double)*2, 1, psSHP->fpSHP );

	if( bBigEndian )
	{
	    SwapWord( 8, &(padBounds->minx) );
	    SwapWord( 8, &(padBounds->miny) );
	}

	padBounds->maxx = padBounds->minx;
	padBounds->maxy = padBounds->miny;
    }
}

int msOpenSHPFile(shapefileObj *shpfile, char *path, char *tile, char *filename)
{
  char *dbfFilename;

  char old_path[MS_PATH_LENGTH];

  getcwd(old_path, MS_PATH_LENGTH); /* save old working directory */
  if(path) chdir(path);
  if(tile) chdir(tile);

  /* ---- open the shapefile file and get basic info ---- */
  shpfile->hSHP = SHPOpen(filename, "rb" );

  if(!shpfile->hSHP) {
    msSetError(MS_IOERR, NULL, "msOpenSHPFile()");
    sprintf(ms_error.message, "(%s)", filename);
    chdir(old_path); /* restore old cwd */
    return(-1);
  }

  if((*filename == '\\') || (*filename == '/')) { /* already full path */
    strcpy(shpfile->source, filename);
  } else {
    getcwd(shpfile->source, MS_PATH_LENGTH); /* save the source (fullpath) information */
    strcat(shpfile->source, "/");
    strcat(shpfile->source, filename);
  }

  /* load some information about this shapefile */
  SHPGetInfo( shpfile->hSHP, &shpfile->numshapes, &shpfile->type);
  msReadBounds( shpfile->hSHP, -1, &(shpfile->extent));
  
  dbfFilename = (char *)malloc(strlen(filename)+5);
  sprintf(dbfFilename, "%s.dbf", filename);
  shpfile->hDBF = DBFOpen(dbfFilename, "rb");
  free(dbfFilename);

  if(!shpfile->hDBF) {
    msSetError(MS_IOERR, NULL, "msOpenSHPFile()");
    sprintf(ms_error.message, "(%s)", dbfFilename);
    chdir(old_path); /* restore old cwd */
    return(-1);
  }
  
  chdir(old_path); /* restore old cwd */

  return(0); /* all o.k. */
}

// Creates a new shapefile
int msCreateSHPFile(shapefileObj *shpfile, char *filename, int type)
{
  if(type != MS_SHP_POINT && type != MS_SHP_MULTIPOINT && type != MS_SHP_ARC && type != MS_SHP_POLYGON) {
    msSetError(MS_SHPERR, "Invalid shape type.", "msNewSHPFile()");
    return(-1);
  }

  // create the spatial portion
  shpfile->hSHP = SHPCreate(filename, type);
  if(!shpfile->hSHP) {
    msSetError(MS_IOERR, NULL, "msNewSHPFile()");
    sprintf(ms_error.message, "(%s)", filename);
    return(-1);
  }

  // retrieve a few things about this shapefile
  SHPGetInfo( shpfile->hSHP, &shpfile->numshapes, &shpfile->type);
  msReadBounds( shpfile->hSHP, -1, &(shpfile->extent));

  shpfile->hDBF = NULL; // XBase file is NOT created here...
  return(0);
}

void msCloseSHPFile(shapefileObj *shpfile)
{
  if(shpfile->hSHP) SHPClose(shpfile->hSHP);
  if(shpfile->hDBF) DBFClose(shpfile->hDBF);
}
