#ifndef MAPSHAPE_H
#define MAPSHAPE_H

#include "mapprimitive.h"
#include "mapproject.h"

#include <shapefil.h>

#ifndef SWIG
#define MS_PATH_LENGTH 1024
#endif

// Shapefile types
#define MS_SHP_POINT 1
#define MS_SHP_ARC 3
#define MS_SHP_POLYGON 5
#define MS_SHP_MULTIPOINT 8

// Shapefile object, no write access via scripts 
typedef struct {
#ifdef SWIG
%readonly
#endif
  char source[MS_PATH_LENGTH]; /* full path to the shapefile data */
  SHPHandle hSHP; /* .SHP/.SHX file pointer */
  int type; /* shape type */
  int numshapes; /* number of shapes */
  rectObj extent; /* shape extent */
  DBFHandle hDBF; /* .DBF file pointer */
#ifdef SWIG
%readwrite
#endif
} shapefileObj;

// Function prototypes, nothing wrapable
#ifndef SWIG
int msWriteShape(SHPHandle psSHP, shapeObj *shape );

#ifdef USE_PROJ
void msReadBoundsProj( SHPHandle psSHP, int hEntity, rectObj *padBounds, projectionObj *in, projectionObj *out );
void msReadShapeProj( SHPHandle psSHP, int hEntity, shapeObj *shape, projectionObj *in, projectionObj *out );
#endif

void msReadBounds( SHPHandle psSHP, int hEntity, rectObj *padBounds );
void msReadShape( SHPHandle psSHP, int hEntity, shapeObj *shape );

int msOpenSHPFile(shapefileObj *shpfile, char *path, char *tile, char *filename);
int msCreateSHPFile(shapefileObj *shpfile, char *filename, int type);
void msCloseSHPFile(shapefileObj *shpfile);
#endif

#endif /* MAPSHAPE_H */
