#include "map.h"
#include "mapindex.h"

int main(int argc, char *argv[])
{
  SHPHandle hSHP;
  int nShapeType;
  int nEntities;
  rectObj shpBounds;
  double tmpBounds[4];
  int i;

  char *filename;
  indexObj *idx;

  if(argc<4) {
   fprintf(stdout,"Syntax: shpindex [shpfile] [sx] [sy]\n" );
   exit(0);
  }
  
  /*
  ** Open shapefile
  */
  if((hSHP = SHPOpen(argv[1], "r" )) == NULL) {
    fprintf(stderr, "Error opening shapefile %s.\n", argv[1]);
    exit(0);
  }

  /* 
  ** Load some information about this shapefile, don't need much
  */
  SHPGetInfo(hSHP, &nEntities, &nShapeType);
  SHPReadBounds(hSHP, -1, tmpBounds);

  shpBounds.minx = tmpBounds[0];
  shpBounds.miny = tmpBounds[1];
  shpBounds.maxx = tmpBounds[2];
  shpBounds.maxy = tmpBounds[3];
  
  /*
  ** Initialize the index
  */
  if((idx = msNewIndex(shpBounds, atoi(argv[2]), atoi(argv[3]), nEntities)) == NULL) {
    msWriteError(stderr);
    exit(0);
  }

  /* 
  ** For each shape 
  */
  for(i=0;i<nEntities;i++) {
    SHPReadBounds(hSHP, i, tmpBounds); /* grab the rectangle for this shape */

    shpBounds.minx = tmpBounds[0];
    shpBounds.miny = tmpBounds[1];
    shpBounds.maxx = tmpBounds[2];
    shpBounds.maxy = tmpBounds[3];

    if(msInsertRect(idx, i, shpBounds) == -1) { /* add it to the index */
      msWriteError(stderr);
      msCloseIndex(idx);
      exit(0);
    }
  }

  filename = (char *)malloc(strlen(argv[1])+strlen(EXTENSION)+1);
  sprintf(filename, "%s%s", argv[1], EXTENSION);
  msWriteIndex(idx, filename);
  msCloseIndex(idx);

  /*
  ** Clean things up
  */
  SHPClose(hSHP);
  free(filename);

  exit(0);
}




